/*************** <lalVerbatim file="VectorFactoriesTestCV"> ****
$Id$
**************** </lalVerbatim> *******************************/

/* <lalLaTeX>

\subsection{Program \texttt{VectorFactoriesTest.c}}
\label{ss:VectorFactoriesTest.c}

A program to test create/destroy vector routines.

\subsubsection*{Usage}
\begin{verbatim}
VectorFactoriesTest [options]
Options:
  -h         print help
  -q         quiet: run silently
  -v         verbose: print extra information
  -d level   set lalDebugLevel to level
\end{verbatim}

\subsubsection*{Description}

\subsubsection*{Exit codes}
\begin{tabular}{|c|l|}
\hline
 Code & Explanation                   \\
\hline
\tt 0 & Success, normal exit.         \\
\tt 1 & Subroutine failed.            \\
\hline
\end{tabular}

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
lalDebugLevel
TYPECODECreateVector()
TYPECODEResizeVector()
TYPECODEDestroyVector()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{VectorFactoriesTestCV}}

</lalLaTeX> */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <lal/LALConfig.h>
#include <lal/PrintVector.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID( MAIN, "$Id$" );

extern char *optarg;
extern int   optind;

extern int lalDebugLevel;
int verbose    = 0;

static void
Usage (const char *program, int exitflag);

static void
ParseOptions (int argc, char *argv[]);

static void
TestStatus (LALStatus *status, const char *expectedCodes, int exitCode);

static void
ClearStatus (LALStatus *status);


/* -*- C -*- */




















static void ZVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static COMPLEX16Vector  *vector = ( COMPLEX16Vector * )NULL;
  static COMPLEX16Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALZCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( COMPLEX16 ) );

  if (verbose)
    {
      LALZPrintVector ( vector );
    }


  /* Resize up */
  LALZResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( COMPLEX16 ) );

  if (verbose)
    {
      LALZPrintVector ( vector );
    }

  /* Resize down */
  LALZResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALZPrintVector ( vector );
    }

  /* Destroy */
  LALZDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALZResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( COMPLEX16 ) );

  if (verbose)
    {
      LALZPrintVector ( vector );
    }

  LALZResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALZCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALZResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALZDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALZCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALZDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALZCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALZDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALZCreateVector, LALZResizeVector, and LALZDestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void CVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static COMPLEX8Vector  *vector = ( COMPLEX8Vector * )NULL;
  static COMPLEX8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALCCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( COMPLEX8 ) );

  if (verbose)
    {
      LALCPrintVector ( vector );
    }


  /* Resize up */
  LALCResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( COMPLEX8 ) );

  if (verbose)
    {
      LALCPrintVector ( vector );
    }

  /* Resize down */
  LALCResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALCPrintVector ( vector );
    }

  /* Destroy */
  LALCDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALCResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( COMPLEX8 ) );

  if (verbose)
    {
      LALCPrintVector ( vector );
    }

  LALCResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALCCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALCResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALCDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALCCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALCDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALCCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALCDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALCCreateVector, LALCResizeVector, and LALCDestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void DVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static REAL8Vector  *vector = ( REAL8Vector * )NULL;
  static REAL8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALDCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL8 ) );

  if (verbose)
    {
      LALDPrintVector ( vector );
    }


  /* Resize up */
  LALDResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( REAL8 ) );

  if (verbose)
    {
      LALDPrintVector ( vector );
    }

  /* Resize down */
  LALDResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALDPrintVector ( vector );
    }

  /* Destroy */
  LALDDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALDResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL8 ) );

  if (verbose)
    {
      LALDPrintVector ( vector );
    }

  LALDResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALDCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALDResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALDDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALDCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALDDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALDCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALDDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALDCreateVector, LALDResizeVector, and LALDDestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void SVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static REAL4Vector  *vector = ( REAL4Vector * )NULL;
  static REAL4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALSCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALSPrintVector ( vector );
    }


  /* Resize up */
  LALSResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALSPrintVector ( vector );
    }

  /* Resize down */
  LALSResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALSPrintVector ( vector );
    }

  /* Destroy */
  LALSDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALSResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALSPrintVector ( vector );
    }

  LALSResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALSCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALSResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALSDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALSCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALSDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALSCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALSDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALSCreateVector, LALSResizeVector, and LALSDestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void I2VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static INT2Vector  *vector = ( INT2Vector * )NULL;
  static INT2Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT2 ) );

  if (verbose)
    {
      LALI2PrintVector ( vector );
    }


  /* Resize up */
  LALI2ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( INT2 ) );

  if (verbose)
    {
      LALI2PrintVector ( vector );
    }

  /* Resize down */
  LALI2ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALI2PrintVector ( vector );
    }

  /* Destroy */
  LALI2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALI2ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT2 ) );

  if (verbose)
    {
      LALI2PrintVector ( vector );
    }

  LALI2ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALI2CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI2ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI2DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI2CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI2DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALI2CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALI2DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALI2CreateVector, LALI2ResizeVector, and LALI2DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void I4VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static INT4Vector  *vector = ( INT4Vector * )NULL;
  static INT4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT4 ) );

  if (verbose)
    {
      LALI4PrintVector ( vector );
    }


  /* Resize up */
  LALI4ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( INT4 ) );

  if (verbose)
    {
      LALI4PrintVector ( vector );
    }

  /* Resize down */
  LALI4ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALI4PrintVector ( vector );
    }

  /* Destroy */
  LALI4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALI4ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT4 ) );

  if (verbose)
    {
      LALI4PrintVector ( vector );
    }

  LALI4ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALI4CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI4ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI4DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI4CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI4DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALI4CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALI4DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALI4CreateVector, LALI4ResizeVector, and LALI4DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void I8VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static INT8Vector  *vector = ( INT8Vector * )NULL;
  static INT8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT8 ) );

  if (verbose)
    {
      LALI8PrintVector ( vector );
    }


  /* Resize up */
  LALI8ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( INT8 ) );

  if (verbose)
    {
      LALI8PrintVector ( vector );
    }

  /* Resize down */
  LALI8ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALI8PrintVector ( vector );
    }

  /* Destroy */
  LALI8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALI8ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( INT8 ) );

  if (verbose)
    {
      LALI8PrintVector ( vector );
    }

  LALI8ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALI8CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI8ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALI8DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI8CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALI8DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALI8CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALI8DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALI8CreateVector, LALI8ResizeVector, and LALI8DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void U2VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static UINT2Vector  *vector = ( UINT2Vector * )NULL;
  static UINT2Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT2 ) );

  if (verbose)
    {
      LALU2PrintVector ( vector );
    }


  /* Resize up */
  LALU2ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( UINT2 ) );

  if (verbose)
    {
      LALU2PrintVector ( vector );
    }

  /* Resize down */
  LALU2ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALU2PrintVector ( vector );
    }

  /* Destroy */
  LALU2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALU2ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT2 ) );

  if (verbose)
    {
      LALU2PrintVector ( vector );
    }

  LALU2ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALU2CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU2ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU2DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU2CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU2DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALU2CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALU2DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALU2CreateVector, LALU2ResizeVector, and LALU2DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void U4VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static UINT4Vector  *vector = ( UINT4Vector * )NULL;
  static UINT4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT4 ) );

  if (verbose)
    {
      LALU4PrintVector ( vector );
    }


  /* Resize up */
  LALU4ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( UINT4 ) );

  if (verbose)
    {
      LALU4PrintVector ( vector );
    }

  /* Resize down */
  LALU4ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALU4PrintVector ( vector );
    }

  /* Destroy */
  LALU4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALU4ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT4 ) );

  if (verbose)
    {
      LALU4PrintVector ( vector );
    }

  LALU4ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALU4CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU4ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU4DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU4CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU4DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALU4CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALU4DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALU4CreateVector, LALU4ResizeVector, and LALU4DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void U8VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static UINT8Vector  *vector = ( UINT8Vector * )NULL;
  static UINT8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT8 ) );

  if (verbose)
    {
      LALU8PrintVector ( vector );
    }


  /* Resize up */
  LALU8ResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( UINT8 ) );

  if (verbose)
    {
      LALU8PrintVector ( vector );
    }

  /* Resize down */
  LALU8ResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALU8PrintVector ( vector );
    }

  /* Destroy */
  LALU8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALU8ResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( UINT8 ) );

  if (verbose)
    {
      LALU8PrintVector ( vector );
    }

  LALU8ResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALU8CreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU8ResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALU8DestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU8CreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALU8DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALU8CreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALU8DestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALU8CreateVector, LALU8ResizeVector, and LALU8DestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void CHARVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static CHARVector  *vector = ( CHARVector * )NULL;
  static CHARVector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALCHARCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( CHAR ) );

  if (verbose)
    {
      LALCHARPrintVector ( vector );
    }


  /* Resize up */
  LALCHARResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( CHAR ) );

  if (verbose)
    {
      LALCHARPrintVector ( vector );
    }

  /* Resize down */
  LALCHARResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALCHARPrintVector ( vector );
    }

  /* Destroy */
  LALCHARDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALCHARResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( CHAR ) );

  if (verbose)
    {
      LALCHARPrintVector ( vector );
    }

  LALCHARResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALCHARCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALCHARResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALCHARDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALCHARCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALCHARDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALCHARCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALCHARDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALCHARCreateVector, LALCHARResizeVector, and LALCHARDestroyVector \n" );

  return;
}



/* -*- C -*- */




















static void VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static LALStatus  status;
  static REAL4Vector  *vector = ( REAL4Vector * )NULL;
  static REAL4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALPrintVector ( vector );
    }


  /* Resize up */
  LALResizeVector ( &status, &vector, length*3 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }


  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALPrintVector ( vector );
    }

  /* Resize down */
  LALResizeVector ( &status, &vector, length*2 );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
      LALPrintVector ( vector );
    }

  /* Destroy */
  LALDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();

  /* Check the resize function with create/destroy */
  LALResizeVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector->length = %d\n", vector->length);
    }

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  if (verbose)
    {
      LALPrintVector ( vector );
    }

  LALResizeVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( 0 ), 1);

  if (verbose)
    {
      printf("VFT line %d:\n", __LINE__);
      printf("  vector = %p\n", (void *)vector);
    }

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */

#ifndef LAL_NDEBUG

  if ( ! lalNoDebug )
  {
    if (verbose)
      {
        printf("VFT line %d\n", __LINE__);
        printf("  vector = %p\n", (void *)vector);
      }

    LALCreateVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALResizeVector ( &status, &vector, 0 );
    TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

    LALDestroyVector ( &status, NULL );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALCreateVector ( &status, NULL, length );
    TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

    LALDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    vector = &vstore;
    LALCreateVector ( &status, &vector, length );
    TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

    LALDestroyVector ( &status, &vector );
    TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );
  }

#else
  vector = &vstore;
#endif

  LALCheckMemoryLeaks();
  printf( "PASS: tests of LALCreateVector, LALResizeVector, and LALDestroyVector \n" );

  return;
}


int main( int argc, char *argv[] )
{
  lalDebugLevel = 0;

  ParseOptions( argc, argv );

  VectorFactoriesTest();
  ZVectorFactoriesTest();
  CVectorFactoriesTest();
  DVectorFactoriesTest();
  SVectorFactoriesTest();
  I2VectorFactoriesTest();
  I4VectorFactoriesTest();
  I8VectorFactoriesTest();
  U2VectorFactoriesTest();
  U4VectorFactoriesTest();
  U8VectorFactoriesTest();
  CHARVectorFactoriesTest();

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (LALStatus *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (LALStatus *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set lalDebugLevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        lalDebugLevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently (ignore error messages) */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}

